<?php
/**
 * @package     Joomla.Site
 * @subpackage  Com_api-plugins
 *
 * @copyright   Copyright (C) 2009-2014 Techjoomla, Tekdi Technologies Pvt. Ltd. All rights reserved.
 * @license     GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link        http://techjoomla.com
 * Work derived from the original RESTful API by Techjoomla (https://github.com/techjoomla/Joomla-REST-API)
 * and the com_api extension by Brian Edgerton (http://www.edgewebworks.com)
 */
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.html.html');

require_once JPATH_ADMINISTRATOR . '/components/com_easysocial/includes/foundry.php';
require_once JPATH_ADMINISTRATOR . '/components/com_easysocial/models/groups.php';
require_once JPATH_ADMINISTRATOR . '/components/com_easysocial/models/covers.php';
require_once JPATH_ADMINISTRATOR . '/components/com_easysocial/models/albums.php';
require_once JPATH_ADMINISTRATOR . '/components/com_easysocial/models/fields.php';
require_once JPATH_SITE . '/plugins/api/easysocial/libraries/mappingHelper.php';
require_once JPATH_SITE . '/plugins/api/easysocial/libraries/uploadHelper.php';

/**
 * API class EasysocialApiResourceLike
 *
 * @since  1.0
 */
class EasysocialApiResourceLike extends ApiResource
{
	/**
	 * Method description
	 *
	 * @return  mixed
	 *
	 * @since 1.0
	 */
	public function get()
	{
		$this->getLikes();
	}

	/**
	 * Method description
	 *
	 * @return  mixed
	 *
	 * @since 1.0
	 */
	public function post()
	{
		$this->toggleLike();
	}

	/**
	 * Method description
	 *
	 * @return  mixed
	 *
	 * @since 1.0
	 */
	public function delete()
	{
		$this->plugin->err_code = 405;
		$this->plugin->err_message = JText::_('PLG_API_EASYSOCIAL_USE_POST_METHOD_MESSAGE');
		$this->plugin->setResponse(null);
	}

	/**
	 * Method function use for get friends data
	 *
	 * @return  mixed
	 *
	 * @since 1.0
	 */
	public function toggleLike()
	{
		// Init variable
		$app		=	JFactory::getApplication();
		$log_user	=	JFactory::getUser($this->plugin->get('user')->id);
		$id			=	$app->input->get('id', 0, 'INT');
		$type		=	$app->input->get('type', null, 'STRING');
		$group		=	$app->input->get('group', 'user', 'STRING');
		$itemVerb	=	$app->input->get('verb', null, 'STRING');
		$streamid	=	$app->input->get('stream_id', 0, 'INT');
		$react		=	$app->input->get('react', 0, 'INT');
		$my			=	FD::user($log_user->id);
		
		if ($react)
		{
			return $this->react();
		}

		$res		=	new stdClass;

		// Load likes library.
		$model		=	FD::model('Likes');

		// Build the key for likes
		$key		=	$type . '.' . $group;

		if ($itemVerb)
		{
			$key	=	$key . '.' . $itemVerb;
		}

		// Determine if user has liked this item previously.
		$hasLiked		=	$model->hasLiked($id, $key, $my->id);
		$useStreamId	=	($type == 'albums') ? '' : $streamid;

		// If user had already liked this item, we need to unlike it.
		if ($hasLiked)
		{
			$state		=	$model->unlike($id, $key, $my->id, $useStreamId);
		}
		else
		{
			$state		=	$model->like($id, $key, $my->id, $useStreamId);

			// Now we need to update the associated stream id from the liked object
			if ($streamid)
			{
				$doUpdate	=	true;

				if ($type == 'photos')
				{
					$sModel		=	FD::model('Stream');
					$totalItem	=	$sModel->getStreamItemsCount($streamid);

					if ($totalItem > 1)
					{
						$doUpdate	=	false;
					}
				}

				if ($doUpdate)
				{
					$stream		=	FD::stream();
					$stream->updateModified($streamid);
				}
			}
		}

		// The current action
		$verb				=	$hasLiked ? JText::_('PLG_API_EASYSOCIAL_UNLIKE') : JText::_('PLG_API_EASYSOCIAL_LIKE');
		$res->result->status		= $state;

		// $res->data		=	($state && $verb == 'like')?$model->getLikesCount($id, $type):0;
		$res->result->message	=	($state)? $verb . JText::_('PLG_API_EASYSOCIAL_SUCCESSFULL'): $verb . JText::_('PLG_API_EASYSOCIAL_UNSUCCESSFULL');

		$this->plugin->setResponse($res);
	}
	
	private function getLikes()
	{
		$app		=	JFactory::getApplication();
		$log_user	=	JFactory::getUser($this->plugin->get('user')->id);
		$stream_id	=	$app->input->get('stream_id', 0, 'INT');
		$type		=	$app->input->get('type', '', 'STRING');
		$filter		=	$app->input->get('filter', '', 'STRING');
		$limit 		= $app->input->get('limit', 10, 'INT');
		$limitstart = $app->input->get('limitstart', 0, 'INT');
		
		
		$tmp = array();
		$res = new stdClass;
		$mapp = new EasySocialApiMappingHelper;
		
		$model = ES::model('Likes');
		$likes = $model->getLikerIds($stream_id, $type, $exclude = array());
		
		if ($limit)
		{
			$likes = array_slice($likes, $limitstart, $limit);
		}

		foreach ($likes as $key => $value) {
			$likes[$key] = $mapp->createUserObj($value);
		}
				
		$res->result = $likes;
		$this->plugin->setResponse($res);
	}
	
	private function react()
	{
		$app = JFactory::getApplication();
		$log_user	=	JFactory::getUser($this->plugin->get('user')->id);
		$id = $app->input->get('id', 0, 'int');
		$type = $app->input->get('type', '', 'string');
		$group = $app->input->get('group', SOCIAL_APPS_GROUP_USER, 'string');
		$verb = $app->input->get('verb', '', 'string');
		$reaction = $app->input->get('reaction', 'like', 'word');
		$streamId = $app->input->get('stream_id', 0, 'int');
		$my			=	FD::user($log_user->id);

		$res = new stdClass;
		
		$key		=	$type . '.' . $group;

		if ($verb)
		{
			$key	=	$key . '.' . $verb;
		}

		// If id is invalid, throw an error.
		/*if (!$id || !$type) {
			return $this->view->exception('COM_EASYSOCIAL_ERROR_UNABLE_TO_LOCATE_ID');
		}
			
		$likes = ES::likes($id, $type, $verb, $group, $streamId, $options);
		$table = $likes->getUserReaction();

		$action = $table->reaction == $reaction ? 'withdraw' : 'react';

		// return $state = $likes->$action($reaction);
		$state = $likes->$action($reaction);*/
		
		//$key = $type.'.'.$group.'.'.$verb;
		$model = ES::model('Likes');
		$state = $model->react($id, $key, $my->id, $reaction, $streamId);

		$res->result = $state;
		$this->plugin->setResponse($res);
		
		// return $this->view->call(__FUNCTION__, $likes, $action);
	}
}
